﻿<#  
.SYNOPSIS  
    JPEG file synchronisation and downsampling
.DESCRIPTION  
    This script 
    - takes all JPEG files from a source and resamples them to a target directory
    - deletes files not existing any longer in source directory
    - resamples newer files in source to target directory
.NOTES  
    File Name  : Sync-Pictures.ps1  
    Author     : Thomas paatsch - data68@live.de
    Version    : 1.04
    Requires   : PowerShell V4
.EXAMPLE
    sync-pictures.ps1 -sourcedir "D:\Bilder" -targetdir c:\users\user123\onedrive\Bilder -xres 900 -yres 600 -mode SYNC
    Only SYNC all JPEGS in numeric directories from D:\Bilder to c:\users\user123\onedrive\Bilder with resolution 900x600
.EXAMPLE
    sync-pictures.ps1 -sourcedir "D:\Bilder" -targetdir c:\users\user123\onedrive\Bilder -mode BOTH -all -filter BACKUP
    SYNC all JPEGS but JPEGS from directories named "BACKUP" from D:\Bilder to c:\users\user123\onedrive\Bilder with current screen resolution and cleanup OneDrive target from missing files
#>

Param(
   # Base path to your pictures source directory (Default: "%userprofile\pictures")
   [Parameter(Mandatory=$false)]
   [string]$SourceDir = [Environment]::GetFolderPath("MyPictures"), 
   # Base path to your pictures target directory (Default: "%userprofile\OneDrive\Bilder")
   [Parameter(Mandatory=$false)]
   [string]$TargetDir = $env:USERPROFILE+"\OneDrive\Bilder",
   # Time in minutes between last write time for resample the target picture (Default: 1)
   [Parameter(Mandatory=$false)]
   [int]$minDelta = 1,
   # Horizontal resolution for the target picture (Default: current x-resolution)
   [Parameter(Mandatory=$false)]
   [int]$xRes = (Get-WmiObject -Class Win32_videocontroller).currenthorizontalresolution,
   # Vertical resolution for the target picture (Default: current y-resolution)
   [Parameter(Mandatory=$false)]
   [int]$yRes = (Get-WmiObject -Class Win32_videocontroller).currentverticalresolution,
   # Filter Directory: (Default: None)
   [Parameter(Mandatory=$false)]
   [string[]]$filters = "$FALSE",
   # Operational mode: SYNC | CLEAN | BOTH (Default: SYNC)
   [Parameter(Mandatory=$false)]
   [string]$mode = "SYNC",
   # If set all source directories will be synchromnized, instead only numerical directory names
   [Parameter(Mandatory=$false)]
   [switch]$all 


)

$SourceDir    = $sourceDir.trimend("\")
$TargetDir  = $TargetDir.trimend("\")
$maxProcesses = 10
$waitTime     = 1
$countSync    = 0
$countClean   = 0

write-host "Syncing with following parameters:"
write-host "=================================="
write-host "Sync-Mode:        $mode"
write-host "Source-Directory: $SourceDir"
write-Host "Target-Directory: $TargetDir"
if ($filters -ne "$false") {
    write-Host "Filter-Directory: $filters"
}
else {
    write-Host "Filter-Directory: None"
}
write-host "Resolution:       $xRes x $yRes"
write-host "Minimal delta:    $minDelta"
write-host "YearsOnly-Mode:  "(-not ($all)) 


# Path definitions and checks
$iviewPath = (Get-itempropertyvalue HKLM:\SOFTWARE\WOW6432Node\IrfanView\shell\open\command -Name "(default)" -ErrorAction SilentlyContinue)
if ($iviewPath) {
    $iviewPath = $iviewPath.replace("""","")
    if (-not (Test-Path ($iviewPath))) {
        write-host "ERROR: IrfanView not installed in $iviewPath" -ForegroundColor "red"
        exit
    } 
}
else {
    write-host "ERROR: IrfanView path not found" -ForegroundColor "red"
    exit

}

if (-not (test-path $SourceDir)) {
    write-host "Source-Directory not found!" -ForegroundColor "Red"
    exit
}
if (-not (test-path $TargetDir)) {
    write-host "Target-Directory not found!" -ForegroundColor "Red"
    $createDir = (Read-Host "Create ""$TargetDir"" (J/N)?").ToUpper()
    if ($createDir -eq "J") {
        new-item $TargetDir -ItemType Directory
    }
    else {
        exit
    }
}


# Chech for numeric value
function Is-Numeric ($Value) {
    return $Value -match "^[\d\.]+$"
}

########
# SYNC #
########

if (($mode -eq "SYNC") -or ($mode -eq "BOTH")) {
    
    $start = get-date

    write-host "#################################"
    write-host "### SYNC to TargetDir started ###"
    write-host "#################################"

    # read Sourcedir 
    if (test-path ($SourceDir)) {
        $dirs = Get-ChildItem -Path $SourceDir -Directory -exclude $filters
    }
    else {
        write-host "Source-Directory not found!" -ForegroundColor "Red"
    }
    
    foreach ($dir in $dirs) {

        # handle only numeric directories names (years), except "-all" is set
        if ( (Is-Numeric $dir.Name) -or ($all) ) {
            Write-Host "Syncing:  $dir (" $dir.FullName ")"
        
 
            # query subdirectories
            $subdirs = @(Get-ChildItem $dir -Directory -Recurse)
            if ($subdirs -eq $null) {
                $subdirs = $dir
            } else
            {
                $subdirs+=$dir
            }
            

            # Step to subdirectories
            foreach ($subdir in $subdirs) {
                write-host "  -> $subdir"
                # Test if Subdir in filter
                $skip=$false
                foreach ($filter in $filters) {
                   #write-host "Check "$subdir.fullname" - $filter"
                   if ((($subdir.fullname).toupper()).contains($filter.toupper())) {
                      write-host " - Filtered: "$subdir.fullname -ForegroundColor "Green"
                      $skip=$true
                      continue
                   }
                }
                if ($skip) {
                    continue
                }

                # Get JPG files by directory
                $files = Get-childitem -path $subdir.FullName -filter *.jpg
        
                foreach ($file in $files) {
            
                    # Define filename in target directory and check if exists
                    $test = $TargetDir+"\"+($file.directoryname).Substring($SourceDir.length + 1)+"\"+$file.name

                    # File exists
                    if (Test-Path ($test)) {

                        # file exists
                        $testfile = Get-ChildItem $test

                        # Source file is newer 
                        if ( ($file.LastWriteTime - $testfile.LastWriteTime).totalminutes -gt $minDelta) {
                    
                            # Define IrfanView paramaters 
                            $parms = """"+$file.FullName+""" /resize=("+$xRes+","+$yRes+") /resample /aspectratio /silent /convert="""+$testfile+""""
                            
                            # if number of processes exceeds $maxProcesses wait $waittime second
                            while ((get-process i_view32 -ErrorAction SilentlyContinue).count -ge $maxProcesses) {
                                sleep $waitTime
                            }

                            # recreate JPEG in target directory
                            $countSync++
                            start-process "C:\Program Files (x86)\IrfanView\i_view32.exe" -ArgumentList $parms
                            write-host " - $file is newer => recreated!"
                        }
                    }

                    # file doesn't exist
                    else {

                        # Define IrfanView paramaters
                        $parms = """"+$file.FullName+""" /resize=("+$xRes+","+$yRes+") /resample /aspectratio /silent /convert="""+$test+""""
                    
                        # if number of processes exceeds $maxProcesses wait $waittime second
                        while ((get-process i_view32 -ErrorAction SilentlyContinue).count -ge $maxProcesses) {
                            sleep $waitTime
                        }

                        # create JPEG in target directory
                        # start-process "C:\Program Files (x86)\IrfanView\i_view32.exe" -ArgumentList $parms -Wait
                        $countSync++
                        start-process "C:\Program Files (x86)\IrfanView\i_view32.exe" -ArgumentList $parms
                        #write-host "-> "$file.fullname
                    }

                }
            }
        }
        # no numeric value
        else {
            write-host "Skipping: $dir"                
        }
    }
    $syncTime = ((get-date) - $start).Seconds
}


#########
# CLEAN #
#########

if (($mode -eq "CLEAN") -or ($mode -eq "BOTH")) {

    $start = (get-date)

    write-host "###############################"
    write-host "### CLEAN TargetDir started ###"
    write-host "###############################"

    # Get JPEG files from target directory 
    if (Test-Path($SourceDir)) {
        $dirs = Get-ChildItem -Path $TargetDir -Directory
        foreach ($dir in $dirs) {

            # Check, if SourceDir exists
            #$testdir = $SourceDir+"\"+($dir.fullname).TrimStart(($TargetDir))
            $testdir = $SourceDir+"\"+($dir.fullname).Substring($TargetDir.length + 1)

            if (test-path $testdir) {
                Write-Host "Cleaning:  $dir ($testdir)"
        
                # Get JPEG files from directories
                $files = Get-childitem -path $dir.FullName -filter *.jpg -Recurse
                foreach ($file in $files) {
            
                    # Check, of file exists in Source
                    $test = $SourceDir+"\"+($file.directoryname).Substring($TargetDir.length +1)+"\"+$file.name
                    if (Test-Path ($test)) {
                        #write-host $test" existiert"
                    }
                    # Doesn't exist => remove from target directory
                    else {
                        write-host " - Missing file $test deleted!"
                        $countClean++
                        Remove-Item $file.fullname
                    }
                }
            }
            # SourceDir doesn't exist
            else {
                write-host "Skipping: $dir"
            }
        }
    }
    else {
        write-host "ERROR: Source-Directory not found!" -foregroundcolor "red"
    }

    $dirs = get-childitem -path $TargetDir -recurse -directory

    foreach ($dir in $dirs) {
        $files = Get-ChildItem -path $dir.fullname
        if ($files -eq $NULL) {
            write-host "Verzeichnis "$dir.fullname" ist leer und wird gelöscht"
            Remove-Item $dir.FullName -force
            $countClean++
        }
    }

    $cleanTime = ((get-date) - $start).Seconds
}


###########
# RESULTS #
###########

write-host "=================================="
if (($mode -eq "SYNC") -or ($mode -eq "BOTH")) {
    write-host "Synced $countSync pictures in $syncTime seconds"
}
if (($mode -eq "CLEAN") -or ($mode -eq "BOTH")) {
    write-host "Cleaned up $countClean pictures/directories in $cleanTime seconds"
}
